import pyodm.model.definition as Model
import pyodm.model.meta.cdisc_odm_entity as Meta


class StudyEventGroupRef(Meta.CdiscODMEntity):
    """
    https://wiki.cdisc.org/display/ODM2/StudyEventGroupRef
    """
    
    StudyEventGroupOID = Model.Attribute()
    
    OrderNumber = Model.Attribute()
    
    Mandatory = Model.Attribute()
    
    CollectionExceptionConditionOID = Model.Attribute()
    
    
    Description = Model.OneElement()
    
